@@ -2,29 +2,23 @@
 #
 # Compact revision graph generator for Mercurial
 #
-# Largely copied from mercurial's own graphlog, which is:
-#   Copyright 2008 Dirkjan Ochtman <dirkjan@ochtman.nl>
-#   Copyright 2007 Joel Rosdahl <joel@rosdahl.net>
-# Modifications for "compact" DAG (squashing uninteresting commit sequences):
+# A modified version of mercurial's own graphlog. Copyright notices for the
+# original files are included below. The modifications are
 #   Copyright 2014 Tikitu de Jager <tikitu@logophile.org>
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-import itertools
 import mercurial
-import mercurial.cmdutil
+from mercurial import cmdutil, commands, scmutil, templatekw, graphmod
+from mercurial.cmdutil import getgraphlogrevs, show_changeset
+from mercurial.graphmod import _fixlongrightedges
 import mercurial.dagutil
 import mercurial.node
-import mercurial.scmutil
-import mercurial.templatekw
-import mercurial.graphmod
-import mercurial.commands
 from mercurial.i18n import _
 
-__version__ = '0.0.2'
-
-SKIPPED = 's'
+testedwith = '2.9'
+__version__ = '0.0.3'
 
 
 class CompactDAG(mercurial.dagutil.genericdag):
@@ -181,6 +175,116 @@ def compactify(dag, repo=None, revs=set(), hide_close_branches=True):
                       backward_parents=backward_parents,
                       interesting=interesting)
 
+# Original: hgext.graphlog
+# ASCII graph log extension for Mercurial
+# Copyright 2007 Joel Rosdahl <joel@rosdahl.net>
+
+cmdtable = {}
+command = cmdutil.command(cmdtable)
+
+
+@command('branchlog',
+    [('f', 'follow', None,
+     _('follow changeset history, or file history across copies and renames')),
+    ('', 'follow-first', None,
+     _('only follow the first parent of merge changesets (DEPRECATED)')),
+    ('d', 'date', '', _('show revisions matching date spec'), _('DATE')),
+    ('C', 'copies', None, _('show copied files')),
+    ('k', 'keyword', [],
+     _('do case-insensitive search for a given text'), _('TEXT')),
+    ('r', 'rev', [], _('show the specified revision or revset'), _('REV')),
+    ('', 'removed', None, _('include revisions where files were removed')),
+    ('m', 'only-merges', None, _('show only merges (DEPRECATED)')),
+    ('u', 'user', [], _('revisions committed by user'), _('USER')),
+    ('', 'only-branch', [],
+     _('show only changesets within the given named branch (DEPRECATED)'),
+     _('BRANCH')),
+    ('b', 'branch', [],
+     _('show changesets within the given named branch'), _('BRANCH')),
+    ('P', 'prune', [],
+     _('do not display revision or any of its ancestors'), _('REV')),
+    ] + commands.logopts + commands.walkopts,
+    _('[OPTION]... [FILE]'))
+def branchlog(ui, repo, *pats, **opts):
+    """show highlights of revision history alongside an ASCII revision graph
+
+    Print a revision history alongside a revision graph drawn with
+    ASCII characters, showing only "interesting" changesets. (What counts
+    as interesting is still under revision.)
+
+    Nodes printed as an @ character are parents of the working
+    directory.
+    """
+    return branchlog_cmd(ui, repo, *pats, **opts)
+
+# Original: mercurial.cmdutil
+# cmdutil.py - help for command processing in mercurial
+# Copyright 2005-2007 Matt Mackall <mpm@selenic.com>
+
+
+def branchlog_cmd(ui, repo, *pats, **opts):
+    # Parameters are identical to log command ones
+    revs, expr, filematcher = getgraphlogrevs(repo, pats, opts)
+    revdag = dagwalker(repo, revs)
+
+    getrenamed = None
+    if opts.get('copies'):
+        endrev = None
+        if opts.get('rev'):
+            endrev = scmutil.revrange(repo, opts.get('rev')).max() + 1
+        getrenamed = templatekw.getrenamedfn(repo, endrev=endrev)
+    displayer = show_changeset(ui, repo, opts, buffered=True)
+    showparents = [ctx.node() for ctx in repo[None].parents()]
+    displaygraph(ui, revdag, displayer, showparents,
+                 graphmod.asciiedges, getrenamed, filematcher)
+
+
+def displaygraph(ui, dag, displayer, showparents, edgefn, getrenamed=None,
+                 filematcher=None):
+    seen, state = [], graphmod.asciistate()
+    for rev, type_, ctx, parents, skipped_count in dag:
+        char = 'o'
+        if type_ == SKIPPED:
+            char = u':'
+        elif ctx.node() in showparents:
+            char = '@'
+        elif ctx.obsolete():
+            char = 'x'
+        copies = None
+        if getrenamed and ctx.rev():
+            copies = []
+            for fn in ctx.files():
+                rename = getrenamed(fn, ctx.rev())
+                if rename:
+                    copies.append((fn, rename[0]))
+        revmatchfn = None
+        if filematcher is not None:
+            revmatchfn = filematcher(ctx.rev())
+        displayer.show(ctx, copies=copies, matchfn=revmatchfn)
+        if type_ == CHANGESET:
+            lines = displayer.hunk.pop(rev).split('\n')
+        elif type_ == SKIPPED:
+            displayer.hunk.pop(rev)
+            lines = ['({} changeset{} {})'.format(
+                skipped_count,
+                's e.g.' if skipped_count != 1 else ':',
+                rev)]
+        if not lines[-1]:
+            del lines[-1]
+        displayer.flush(rev)
+        edges = edgefn(type_, char, lines, seen, rev, parents)
+        for type_, char, lines, coldata in edges:
+            ascii(ui, state, type_, char, lines, coldata)
+    displayer.close()
+
+# Original: mercurial.graphmod
+# Revision graph generator for Mercurial
+# Copyright 2008 Dirkjan Ochtman <dirkjan@ochtman.nl>
+# Copyright 2007 Joel Rosdahl <joel@rosdahl.net>
+
+CHANGESET = 'C'
+SKIPPED = 's'
+
 
 def dagwalker(repo, revs):
     """cset DAG generator yielding (id, CHANGESET, ctx, [parentids]) tuples
@@ -198,7 +302,7 @@ def dagwalker(repo, revs):
                      revs=revs)
     revs = sorted(filter(None, set(dag.compacted(rev) for rev in revs)), reverse=True)
 
-    get_ancestors = mercurial.graphmod.grandparent
+    get_ancestors = graphmod.grandparent
 
     changelog = repo.changelog
     lowest_rev = min(revs)
@@ -222,65 +326,73 @@ def dagwalker(repo, revs):
         parents = sorted(set(parents))
 
         if dag.interesting(rev):
-            yield (ctx.rev(), mercurial.graphmod.CHANGESET, ctx, parents, None)
+            yield (ctx.rev(), graphmod.CHANGESET, ctx, parents, None)
         else:
             yield (ctx.rev(), SKIPPED, ctx, parents, dag.skip_counts[rev])
 
 
-def displaygraph(ui, dag, displayer, showparents, edgefn, getrenamed=None,
-                 filematcher=None):
-    seen, state = [], mercurial.graphmod.asciistate()
-    for rev, type_, ctx, parents, skipped_count in dag:
-        char = 'o'
-        if type_ == SKIPPED:
-            char = u':'
-        elif ctx.node() in showparents:
-            char = '@'
-        elif ctx.obsolete():
-            char = 'x'
-        copies = None
-        if getrenamed and ctx.rev():
-            copies = []
-            for fn in ctx.files():
-                rename = getrenamed(fn, ctx.rev())
-                if rename:
-                    copies.append((fn, rename[0]))
-        revmatchfn = None
-        if filematcher is not None:
-            revmatchfn = filematcher(ctx.rev())
-        displayer.show(ctx, copies=copies, matchfn=revmatchfn)
-        if type_ == mercurial.graphmod.CHANGESET:
-            lines = displayer.hunk.pop(rev).split('\n')
-        elif type_ == SKIPPED:
-            displayer.hunk.pop(rev)
-            lines = ['({} changeset{} {})'.format(
-                skipped_count,
-                's e.g.' if skipped_count != 1 else ':',
-                rev)]
-        if not lines[-1]:
-            del lines[-1]
-        displayer.flush(rev)
-        edges = edgefn(type_, char, lines, seen, rev, parents)
-        for type_, char, lines, coldata in edges:
-            ascii(ui, state, type_, char, lines, coldata)
-    displayer.close()
+def _getnodelineedgestail(
+        node_index, p_node_index, n_columns, n_columns_diff, p_diff, fix_tail):
+    if fix_tail and n_columns_diff == p_diff and n_columns_diff != 0:
+        # Still going in the same non-vertical direction.
+        if n_columns_diff == -1:
+            start = max(node_index + 1, p_node_index)
+            tail = ["|", " ", ' '] * (start - node_index - 1)
+            tail.extend(["/", " ", ' '] * (n_columns - start))
+            return tail
+        else:
+            return ["\\", " ", ' '] * (n_columns - node_index - 1)
+    else:
+        return ["|", " ", ' '] * (n_columns - node_index - 1)
 
 
-def branchlog_cmd(ui, repo, *pats, **opts):
-    # Parameters are identical to log command ones
-    revs, expr, filematcher = mercurial.cmdutil.getgraphlogrevs(repo, pats, opts)
-    revdag = dagwalker(repo, revs)
+def _drawedges(edges, nodeline, interline):
+    needs_extra_interline = False
+    for (start, end) in edges:
+        if start == end + 1:
+            interline[3 * end + 2] = "/"
+            needs_extra_interline = True
+        elif start == end - 1:
+            interline[3 * start + 1] = "\\"
+            needs_extra_interline = True
+        elif start == end:
+            interline[3 * start] = "|"
+        else:
+            if 3 * end >= len(nodeline):
+                continue
+            nodeline[3 * end] = "|"
+            step = 1 if start < end else -1
+            for i in range(3 * start + step, 3 * end - step, step):
+                if nodeline[i] == ' ':
+                    nodeline[i] = "-"
+    return needs_extra_interline
 
-    getrenamed = None
-    if opts.get('copies'):
-        endrev = None
-        if opts.get('rev'):
-            endrev = mercurial.scmutil.revrange(repo, opts.get('rev')).max() + 1
-        getrenamed = mercurial.templatekw.getrenamedfn(repo, endrev=endrev)
-    displayer = mercurial.cmdutil.show_changeset(ui, repo, opts, buffered=True)
-    showparents = [ctx.node() for ctx in repo[None].parents()]
-    displaygraph(ui, revdag, displayer, showparents,
-                 mercurial.graphmod.asciiedges, getrenamed, filematcher)
+
+def _getpaddingline(ni, n_columns, edges):
+    line = []
+    line.extend(["|", " ", ' '] * ni)
+    if (ni, ni - 1) in edges or (ni, ni) in edges:
+        # (ni, ni - 1)      (ni, ni)
+        # | | | |           | | | |
+        # +---o |           | o---+
+        # | | c |           | c | |
+        # | |/ /            | |/ /
+        # | | |             | | |
+        c = "|"
+    else:
+        c = " "
+    line.extend([c, " ", ' '])
+    line.extend(["|", " ", ' '] * (n_columns - ni - 1))
+    return line
+
+
+def _shift(shift_interline):
+    for (a, b, c) in zip(shift_interline[::3],
+                         shift_interline[1::3],
+                         shift_interline[2::3]):
+        yield '|' if a == '|' else ' '
+        yield '/' if c == '/' else ' '
+        yield '\\' if b == '\\' else ' '
 
 
 def ascii(ui, state, type, char, text, coldata):
@@ -306,14 +418,14 @@ def ascii(ui, state, type, char, text, coldata):
 
     idx, edges, ncols, coldiff = coldata
     assert -2 < coldiff < 2
-    if coldiff == -1 and False:
+    if coldiff == -1 and False:  # wide columns mean we can happily cross
         # Transform
         #
         #     | | |        | | |
         #     o | |  into  o---+
         #     |X /         |/ /
         #     | |          | |
-        mercurial.graphmod._fixlongrightedges(edges)
+        _fixlongrightedges(edges)
 
     # add_padding_line says whether to rewrite
     #
@@ -394,111 +506,3 @@ def ascii(ui, state, type, char, text, coldata):
     # ... and start over
     state[0] = coldiff
     state[1] = idx
-
-
-def _shift(shift_interline):
-    for (a, b, c) in zip(shift_interline[::3],
-                         shift_interline[1::3],
-                         shift_interline[2::3]):
-        yield '|' if a == '|' else ' '
-        yield '/' if c == '/' else ' '
-        yield '\\' if b == '\\' else ' '
-
-
-def _getnodelineedgestail(
-        node_index, p_node_index, n_columns, n_columns_diff, p_diff, fix_tail):
-    if fix_tail and n_columns_diff == p_diff and n_columns_diff != 0:
-        # Still going in the same non-vertical direction.
-        if n_columns_diff == -1:
-            start = max(node_index + 1, p_node_index)
-            tail = ["|", " ", ' '] * (start - node_index - 1)
-            tail.extend(["/", " ", ' '] * (n_columns - start))
-            return tail
-        else:
-            return ["\\", " ", ' '] * (n_columns - node_index - 1)
-    else:
-        return ["|", " ", ' '] * (n_columns - node_index - 1)
-
-
-def _drawedges(edges, nodeline, interline):
-    needs_extra_interline = False
-    for (start, end) in edges:
-        if start == end + 1:
-            interline[3 * end + 2] = "/"
-            needs_extra_interline = True
-        elif start == end - 1:
-            interline[3 * start + 1] = "\\"
-            needs_extra_interline = True
-        elif start == end:
-            interline[3 * start] = "|"
-        else:
-            if 3 * end >= len(nodeline):
-                continue
-            nodeline[3 * end] = "|"
-            step = 1 if start < end else -1
-            for i in range(3 * start + step, 3 * end - step, step):
-                if nodeline[i] == ' ':
-                    nodeline[i] = "-"
-    return needs_extra_interline
-
-
-def _getpaddingline(ni, n_columns, edges):
-    line = []
-    line.extend(["|", " ", ' '] * ni)
-    if (ni, ni - 1) in edges or (ni, ni) in edges:
-        # (ni, ni - 1)      (ni, ni)
-        # | | | |           | | | |
-        # +---o |           | o---+
-        # | | c |           | c | |
-        # | |/ /            | |/ /
-        # | | |             | | |
-        c = "|"
-    else:
-        c = " "
-    line.extend([c, " ", ' '])
-    line.extend(["|", " ", ' '] * (n_columns - ni - 1))
-    return line
-
-
-cmdtable = {}
-command = mercurial.cmdutil.command(cmdtable)
-testedwith = '2.9'
-
-@command(
-    'branchlog',
-    [('f', 'follow', None,
-      _('follow changeset history, or file history across copies and renames')),
-     ('', 'follow-first', None,
-      _('only follow the first parent of merge changesets (DEPRECATED)')),
-     ('d', 'date', '', _('show revisions matching date spec'), _('DATE')),
-     ('C', 'copies', None, _('show copied files')),
-     ('k', 'keyword', [],
-      _('do case-insensitive search for a given text'), _('TEXT')),
-     ('r', 'rev', [], _('show the specified revision or revset'),
-      _('REV')),
-     ('', 'removed', None,
-      _('include revisions where files were removed')),
-     ('m', 'only-merges', None, _('show only merges (DEPRECATED)')),
-     ('u', 'user', [], _('revisions committed by user'), _('USER')),
-     ('', 'only-branch', [],
-      _('show only changesets within the given named branch (DEPRECATED)'),
-      _('BRANCH')),
-     ('b', 'branch', [],
-      _('show changesets within the given named branch'), _('BRANCH')),
-     ('P', 'prune', [],
-      _('do not display revision or any of its ancestors'), _('REV')),
-    ] + mercurial.commands.logopts + mercurial.commands.walkopts,
-    _('[OPTION]... [FILE]'))
-def graphlog(ui, repo, *pats, **opts):
-    """show highlights of revision history alongside an ASCII revision graph
-
-    Print a revision history alongside a revision graph drawn with
-    ASCII characters, showing only "interesting" changesets. (What counts
-    as interesting is still under revision.)
-
-    Nodes printed as an @ character are parents of the working
-    directory.
-    """
-    return branchlog_cmd(ui, repo, *pats, **opts)
-
-
